local GetPlayer = GLOBAL.GetPlayer
local TheInput = GLOBAL.TheInput
local TheSim = GLOBAL.TheSim

local attackTarget = nil
local attackTargetDuration = 0
local attackRange = 8

local function TryGetNearbyTarget(pc)
	local x, y, z = GetPlayer().Transform:GetWorldPosition()
	local ents = TheSim:FindEntities(x, y, z, attackRange, nil)

	for k, v in pairs(ents) do
		if v ~= GetPlayer() and 
			v.components.health 
			and not v.components.health:IsDead() 
			and not GetPlayer().components.leader:IsFollower(v) 
			and not v:HasTag("companion") 
			and not v:HasTag("INLIMBO") 
			and not (v:HasTag("shadow") and not v.components.combat:CanBeAttacked(GetPlayer()))
			then
				pc.controller_attack_target = v
				attackTarget = v
				attackTargetDuration = 10
				return true
			end
	end
	
	-- Still nil? *Then* don't attack anything
	if attackTarget == nil then
		return false
	end

	return true
end

AddClassPostConstruct("widgets/controllercrafting", function(self)
	local _OnControl = self.OnControl

	function self:OnControl(control, down)
		if (GetModConfigData("fixCraftHud")) == false then
			_OnControl(self, control, down)
			return
		end

		if not self.open then return end
	
		if not down and (control == GLOBAL.CONTROL_ACCEPT or control == GLOBAL.CONTROL_ACTION) then
			if self.accept_down then
				self.accept_down = false
			else
				if not GLOBAL.DoRecipeClick(self.owner, self.selected_recipe_by_tab_idx[self.tabidx], true) then 
					self.owner.HUD:CloseControllerCrafting() -- Do close if we're placing a building down, or making a new recipe
				end
				-- Don't close ya buffoon
				if self.owner.components.builder:CanBuild(self.selected_recipe_by_tab_idx[self.tabidx].name) then
					GLOBAL.TheFrontEnd:GetSound():PlaySound("dontstarve/HUD/click_move")
				else
					GLOBAL.TheFrontEnd:GetSound():PlaySound("dontstarve/HUD/click_negative")
				end
			end

			return true
		end
	
		if not down and control == CONTROL_OPEN_CRAFTING and self.control_held and self.control_held_time > 1 then
			self.owner.HUD:CloseControllerCrafting()
			return true
		end
	end
end)

AddPrefabPostInit("trap_teeth", function(inst)
	if (GetModConfigData("shovelTraps")) == false then
		return
	end

	local function IsDropped()
		inst.components.workable:SetWorkAction(GLOBAL.ACTIONS.BLANK)
		inst.components.inventoryitem.canbepickedup = true
	end
	local function IsPlanted()
		inst.components.workable:SetWorkAction(GLOBAL.ACTIONS.DIG)
		inst.components.inventoryitem.canbepickedup = false
	end
	local function OnDropped()
		inst.components.mine:Deactivate()
		IsDropped()
	end
	local function OnDeploy(inst, pt, deployer)
		IsPlanted()
		inst.components.workable:SetWorkLeft(1)
		inst.components.mine:Reset()
		inst.Physics:Teleport(pt:Get())
	end
	local function OnLoad(inst, data)
		if not data or not data.mine then
			inst.components.mine:Reset()
		end
		if inst.components.mine.inactive then
			IsDropped()
		else
			IsPlanted()
		end
	end
	local function OnFinishCallBack()
		inst.components.mine:Deactivate()
		IsDropped()
	end

	inst.components.deployable.ondeploy = OnDeploy
	
	inst.components.inventoryitem:SetOnDroppedFn(OnDropped)

	inst.OnLoad = OnLoad

	inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(GLOBAL.ACTIONS.DIG)
    inst.components.workable:SetWorkLeft(1)
	inst.components.workable:SetOnFinishCallback(OnFinishCallBack)
end)

AddComponentPostInit("playercontroller", function(self)
	function self:UsingMouse() -- Only checks for controllers in here anyway
		return true
	end
end)

local function AddGamepadImprovements(inst)
	-- Based on the original mod by pigshaam
	if TheInput.ControllerAttached then
		TheInput.EnableMouse = function(self, enable)
			self.mouse_enabled = true
		end
	else
		return
	end

	local mouseEnabled = false
	local pc = inst.components.playercontroller

  	-- Modifying controller attack here
  	local _ControllerAttack = pc.DoControllerAttack
	pc.DoControllerAttack = function(self)
		local pcTarget = pc.controller_attack_target
		if pcTarget == nil then
			if attackTarget ~= nil and not attackTarget:IsInLimbo() then
				-- Try to get a nearby target
				local targetInRange = false
				local x, y, z = GetPlayer().Transform:GetWorldPosition()
				local ents = TheSim:FindEntities(x, y, z, attackRange, nil)

				for k, v in pairs(ents) do
					if v == attackTarget then 
						targetInRange = true

						attackTarget:ListenForEvent("death", function(trgt) 
							if trgt == attackTarget then
								attackTarget = nil
							end
						end)
						attackTarget:ListenForEvent("onremove", function(trgt) 
							if trgt == attackTarget then
								attackTarget = nil
							end
						end)
		
						pc.controller_attack_target = attackTarget
						attackTargetDuration = 10
					end
				end

				-- Don't do anything if our saved target isn't in range
				if not targetInRange then
					return
				else
					if not TryGetNearbyTarget(pc) then
						return
					end
				end
			else
				attackTarget = nil
				if not TryGetNearbyTarget(pc) then
					return
				end
			end
		end

		pcTarget = pc.controller_attack_target
		if (GetModConfigData("attackCompanions")) == false and pcTarget and (pcTarget:HasTag("companion") or GetPlayer().components.leader:IsFollower(pcTarget)) then
			TryGetNearbyTarget(pc)
			return
		end

		_ControllerAttack(pc)
	end

	local _ControllerAttached = TheInput.ControllerAttached
	TheInput.ControllerAttached = function(self)
		if mouseEnabled then
			return false
		end
		return _ControllerAttached(self)
	end

	local function UpdateMouseHighlight()
		if not mouseEnabled then
			return
		end

		local new_highlight = nil
		pc.LMBaction, pc.RMBaction = pc.inst.components.playeractionpicker:DoGetMouseActions()
		new_highlight = (pc.LMBaction and pc.LMBaction.target) or (pc.RMBaction and pc.RMBaction.target)
  
		if new_highlight ~= pc.highlight_mouse then
			if pc.highlight_mouse and pc.highlight_mouse:IsValid() then
				if pc.highlight_mouse.components.highlight then
					pc.highlight_mouse.components.highlight:UnHighlight()
				end
			end
			pc.highlight_mouse = new_highlight
		end
  
		if pc.highlight_mouse and pc.highlight_mouse:IsValid() then
			if not pc.highlight_mouse.components.highlight then
				pc.highlight_mouse:AddComponent("highlight")
			end
  
			local override = pc.highlight_mouse.highlight_override
			if override then
				pc.highlight_mouse.components.highlight:Highlight(override[1], override[2], override[3])
			else
				pc.highlight_mouse.components.highlight:Highlight()
			end
		else
			pc.highlight_mouse = nil
		end
	end

	local _OnMouseMove = TheInput.OnMouseMove
	TheInput.OnMouseMove = function(self, x, y)
		if not mouseEnabled then
			return
		end
		_OnMouseMove(self, x, y)
		UpdateMouseHighlight()
  	end

  	local _OnControl = TheInput.OnControl
  	TheInput.OnControl = function(self, control, digitalvalue, analogvalue)
		if control == GLOBAL.CONTROL_PRIMARY or control == GLOBAL.CONTROL_SECONDARY then
			pc.inst:DoTaskInTime(0, function() 
				mouseEnabled = true
			end)
		end
		if 	control == GLOBAL.CONTROL_OPEN_INVENTORY or
			control == GLOBAL.CONTROL_OPEN_CRAFTING or
			control == GLOBAL.CONTROL_INVENTORY_LEFT or
			control == GLOBAL.CONTROL_INVENTORY_RIGHT or
			control == GLOBAL.CONTROL_INVENTORY_UP or
			control == GLOBAL.CONTROL_INVENTORY_DOWN or
			control == GLOBAL.CONTROL_CONTROLLER_ATTACK or
			control == GLOBAL.CONTROL_CONTROLLER_ACTION or
			control == GLOBAL.CONTROL_CONTROLLER_ALTACTION then
				if mouseEnabled and pc.highlight_mouse and pc.highlight_mouse.components.highlight and pc.highlight_mouse.components.highlight.highlit then
					pc.highlight_mouse.components.highlight:UnHighlight()
				end
				mouseEnabled = false
				pc.LMBaction = nil
				pc.RMBaction = nil
		end
		_OnControl(self, control, digitalvalue, analogvalue)
	end

	pc.OnLeftClick = function(self, down)
		if not self:UsingMouse() then return end
    
		if not down then return self:OnLeftUp() end
	
		self.startdragtime = nil
	
		if not self:IsEnabled() then return end
		
		if TheInput:GetHUDEntityUnderMouse() then 
			self:CancelPlacement()
			return 
		end
	
		if self.placer_recipe and self.placer then
			--do the placement
			if self.placer.components.placer.can_build then
				local pos = self.placer.components.placer.targetPos or TheInput:GetWorldPosition()
				self.inst.components.builder:MakeRecipe(self.placer_recipe, TheInput:GetWorldPosition())
				
				self:CancelPlacement()
			end
			return
		end
		
		self.inst.components.combat.target = nil
		
		if self.inst.inbed then
			self.inst.inbed.components.bed:StopSleeping()
			return
		end
		
		local action = self:GetLeftMouseAction()
		if action then
			self:DoAction(action)
		else
			if not mouseEnabled then
				return
			end

			self:DoAction(GLOBAL.BufferedAction(self.inst, nil, GLOBAL.ACTIONS.WALKTO, nil, TheInput:GetWorldPosition())) 		
			local clicked = TheInput:GetWorldEntityUnderMouse()
			if not clicked then
				self.startdragtime = GLOBAL.GetTime()
			end
		end
	end

  	local function StartUpdating()
        inst:DoPeriodicTask(1, function() 
			attackTargetDuration = attackTargetDuration - 1 

			if attackTargetDuration <= 0 then
				attackTarget = nil
			end

			UpdateMouseHighlight()
		end)
  	end

  	StartUpdating()
end

-- Moar save slots left/right controls
if GLOBAL.KnownModIndex:IsModEnabled("workshop-333826502") then
	AddClassPostConstruct("screens/loadgamescreen", function(self) 
		local _OnControl = self.OnControl
		function self:OnControl(control, down)
			if not down and (control == GLOBAL.CONTROL_INVENTORY_USEONSELF or control == GLOBAL.CONTROL_MOVE_RIGHT) then
				GLOBAL.TheFrontEnd:GetSound():PlaySound("dontstarve/HUD/click_move")	
				self:Scroll(self.controls_per_screen)
				self:OnBecomeActive()
				return true
			elseif not down and (control == GLOBAL.CONTROL_INVENTORY_USEONSCENE or control == GLOBAL.CONTROL_MOVE_LEFT) then
				GLOBAL.TheFrontEnd:GetSound():PlaySound("dontstarve/HUD/click_move")	
				self:Scroll(-self.controls_per_screen)
				self:OnBecomeActive()
				return true
			end
			_OnControl(self, control, down)
		end
	end)
end

-- Better Mods Screen controller support
if GLOBAL.KnownModIndex:IsModEnabled("workshop-2842240212") then
	AddGlobalClassPostConstruct("screens/modsscreen", "ModsScreen", function(self)
		self.cancelbutton:SetFocusChangeDir(GLOBAL.MOVE_LEFT, self.applybutton)
		self.cancelbutton:SetFocusChangeDir(GLOBAL.MOVE_RIGHT, self.morebutton)
		self.cancelbutton:SetFocusChangeDir(GLOBAL.MOVE_UP, self.optionspanel)
		self.cancelbutton:SetFocusChangeDir(GLOBAL.MOVE_DOWN, self.optionspanel)
		
		self.morebutton:SetFocusChangeDir(GLOBAL.MOVE_LEFT, self.cancelbutton)
		self.morebutton:SetFocusChangeDir(GLOBAL.MOVE_UP, self.optionspanel)
		self.morebutton:SetFocusChangeDir(GLOBAL.MOVE_DOWN, self.optionspanel)
		
		self.applybutton:SetFocusChangeDir(GLOBAL.MOVE_LEFT, self.optionspanel)
		self.applybutton:SetFocusChangeDir(GLOBAL.MOVE_UP, self.optionspanel)
		self.applybutton:SetFocusChangeDir(GLOBAL.MOVE_DOWN, self.optionspanel)
		
		self.leftbutton:SetFocusChangeDir(GLOBAL.MOVE_RIGHT, self.applybutton)
		self.leftbutton:SetFocusChangeDir(GLOBAL.MOVE_DOWN, self.optionspanel)
		self.rightbutton:SetFocusChangeDir(GLOBAL.MOVE_RIGHT, self.applybutton)
		self.rightbutton:SetFocusChangeDir(GLOBAL.MOVE_DOWN, self.optionspanel)
		
		local _OnControl = self.OnControl
		function self:OnControl(control, down)
			if self.modconfigbutton:IsVisible() then
				self.optionspanel:SetFocusChangeDir(GLOBAL.MOVE_RIGHT, self.modconfigbutton)
			else
				self.optionspanel:SetFocusChangeDir(GLOBAL.MOVE_RIGHT, self.applybutton)
			end
			return _OnControl(self, control, down)
		end
	end)
end

AddSimPostInit(AddGamepadImprovements)